<?php
/**
 * Scand_ProductLocator extension is a extension to display specific products on Product Locator.
 * Copyright (C) 2013 Scand Ltd.
 * 
 * This file is part of Scand_ProductLocator extension.
 * 
 * Scand_ProductLocator extension is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Scand_ProductLocator extension is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Scand_ProductLocator extension. If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Scand
 * @package     Scand_ProductLocator
 * @copyright   Copyright (c) 2013 Scand Ltd. (http://www.scand.com)
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU General Public License (GPL 3.0)
 */
class Scand_ProductLocator_Block_Product_Options extends Mage_Core_Block_Template
{
	private $_options = array();
	private $_uniqueProducts = array();
	
	public function __construct() {
		parent::__construct();
	}

	public function getOptions() {
		$bGroupByCity = Mage::getStoreConfig('productlocator/api/group_by') == 'city';

		$collection = Mage::getModel('productlocator/product_address')
			->getCollection()
			->addFieldToSelect('country_id')
			->addFieldToSelect('region_id')
			->addFieldToSelect('region')
			->addFieldToSelect('city')
			->addFieldToSelect('postcode')
			->addFieldToSelect('street')
			->addFieldToSelect('telephone')
			->addFieldToSelect('gmaps_lat')
			->addFieldToSelect('gmaps_lng')
			->setPageSize(100);
		
		$collection->getSelect()->join(
			array('cpa' => $this->_getTable('catalog/product')),
			'cpa.entity_id = main_table.product_id',
			array('main_table.product_id')
		);

		if (!$bGroupByCity) {
			$collection->getSelect()->join(
				array('ccp' => $this->_getTable('catalog/category_product')),
				'ccp.product_id = cpa.entity_id',
				array('ccp.category_id')
			);
		}
		
		$this->_uniqueProducts = array();
		foreach ($collection as $address) {
			$product = Mage::getModel('catalog/product')
				->load($address->getProductId());

			if ($bGroupByCity) {
				$city = preg_replace('#[^0-9a-z]+#i', '-', $address->getCity());
				$city = trim(strtolower($city), '-');

				$region_code = '';
				if ($address->getCountryId() == 'US' && intval($address->getRegionId()) > 0) {
					$region = Mage::getModel('directory/region')
						->load($address->getRegionId());
					$region_code = $region->getCode().', ';
				}

				/*$country_code = $address->getCountryId();
				if ($country_code <> '')
					$country_code .= ', ';*/

				$option_id = $address->getCountryId().'_'.$address->getRegionId().'_'.$city;
				$option_name = /*$country_code.*/$region_code.$address->getCity();
			}
			else {
				$category = Mage::getModel('catalog/category')
					->load($address->getCategoryId());
				$option_id = $category->getId();
				$option_name = $category->getName();
			}
			if (!array_key_exists($option_id, $this->_options))
				$this->_options[$option_id] = array('name' => $option_name, 'products' => array());
			
			$this->_uniqueProducts[$product->getId()] = 1;
			$this->_options[$option_id]['products'][] = array(
				'product_name' => $product->getName(),
				'product_id' => $product->getId(),
				'product_url' => $product->getProductUrl(),
				'address' => Mage::helper('productlocator/data')->getAddress($address),
				'phone' => $address->getTelephone(),
				'lat' => (float)$address->getGmapsLat(),
				'lng' => (float)$address->getGmapsLng()
			);
		}
		
		return $this->_options;
	}
	
	public function getProductAddressesJson() {
		$product_addresses = array();
		foreach ($this->_options as $option_id => $data) {
			$product_addresses[$option_id] = $data['products'];
		}
		
		return empty($product_addresses) ? "{}" : Mage::helper('core')->jsonEncode($product_addresses);
	}
	
	public function getAllProductsCount() {
		return sizeof(array_keys($this->_uniqueProducts));
	}
	
	private function _getTable($table) {
		return Mage::getSingleton('core/resource')->getTableName($table);
	}
}