/**
 * Scand_ProductLocator extension is a extension to display specific products on Product Locator.
 * Copyright (C) 2013 Scand Ltd.
 * 
 * This file is part of Scand_ProductLocator extension.
 * 
 * Scand_ProductLocator extension is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Scand_ProductLocator extension is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Scand_ProductLocator extension. If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Scand
 * @package     Scand_ProductLocator
 * @copyright   Copyright (c) 2013 Scand Ltd. (http://www.scand.com)
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU General Public License (GPL 3.0)
 */
var ScandProductLocator = new Class.create();
ScandProductLocator.prototype = {
	initialize : function(data){
	if (!data) data = {};
		// Google objects
		this.map = null;
		this.geocoder = new google.maps.Geocoder();
		this.infowindow = new google.maps.InfoWindow();
		// Scand Ltd. position by default
		this.defaultLat = data.lat ? data.lat : 53.889713;
		this.defaultLng = data.lng ? data.lng : 27.536919;
		// count of unique products
		this.productCount = data.count ? data.count : 0;
		// list af addresses
		this.productAddresses = data.addresses ? data.addresses : {};
		// category to display by default
		this.defaultOption = data.option ? data.option : 0;
		// arrays of positions to calculate center of map
		this.lats = [];
		this.lngs = [];
		// list of google.maps.Marker
		this.markers = [];
		// count of horizontal/vertical cells
		this.xCells = 13;
		this.yCells = 10;

		this.currentOption = null;

		this.validate();
	},
	
	validate : function () {
		if (this.productCount == 0)
			return;

		var lats = [],
			lngs = [];

		for (var opt_id in this.productAddresses) {
			if (this.productAddresses.hasOwnProperty(opt_id)) {
				this.productAddresses[opt_id].map(function(product){
					if (product.address && !product.lat && !product.lng) {
						this.geocoder.geocode({'address': product.address.replace(/\<br\>/g, ", ")}, function(results, status) {
							if (status == google.maps.GeocoderStatus.OK) {
								var item = results.first();
								product.lat = item.geometry.location.lat();
								product.lng = item.geometry.location.lng();
								lats.push(product.lat);
								lngs.push(product.lng);
								//this.addMarker(product);
							}
						});
					}
				}, this);
			}
		}
	},
	
	addProduct : function (opt_id, productsToShow) {
		this.productAddresses[opt_id].collect(function(product){
			if ((product.lat && product.lng) || (product.address !="")) {
				productsToShow[product.product_id] = product;
				this.lats.push(product.lat);
				this.lngs.push(product.lng);
			}
		}, this);
		return productsToShow;
	},
	
	showProductsByOption : function (opt_id, no_clear, no_resize) {
		var gridMarkers = {},
			productsToShow = {};
		this.currentOption = opt_id;

		if (!no_clear) {
			$$('#productlocator_categories li a').each(function(e){
				e.setStyle({fontWeight: e === $('cat_'+opt_id) ? 'bold' : 'normal'});
			});

			var i = 0;
			for (i; i < this.markers.length; i++) {
				this.markers[i].setMap(null);
			}
			
			this.markers = [];
			this.lats = [];
			this.lngs = [];
		}
		
		if (this.productCount == 0) {
			return;
		}
		else if (opt_id == 0) {
			for (var opt_id in this.productAddresses) {
				productsToShow = this.addProduct(opt_id, productsToShow);
			}
		}
		else if (this.productAddresses.hasOwnProperty(opt_id)) {
			productsToShow = this.addProduct(opt_id, productsToShow);
		}

		if(this.map.getBounds()) {
			var max_lat = this.map.getBounds().getNorthEast().lat(),
				min_lat = this.map.getBounds().getSouthWest().lat(),
				max_lng = this.map.getBounds().getNorthEast().lng(),
				min_lng = this.map.getBounds().getSouthWest().lng();
		}
		else {
			var min_lat = this.lats.min()-5, max_lat = this.lats.max()+5,
				min_lng = this.lngs.min()-5, max_lng = this.lngs.max()+5;
		}

		var lat_dim = (max_lat - min_lat) / this.xCells;
		var lng_dim = (max_lng - min_lng) / this.yCells;
		
		for (var ndx in productsToShow) {
			if (productsToShow.hasOwnProperty(ndx)) {
				var product = productsToShow[ndx];
				var cell_lng_ind = Math.ceil((product.lng - min_lng) / lng_dim);
				var cell_lat_ind = Math.ceil((product.lat - min_lat) / lat_dim);

				if (!gridMarkers[cell_lng_ind])
					gridMarkers[cell_lng_ind] = {};
				if (!gridMarkers[cell_lng_ind][cell_lat_ind])
					gridMarkers[cell_lng_ind][cell_lat_ind] = {};
				if (!gridMarkers[cell_lng_ind][cell_lat_ind]['products'])
					gridMarkers[cell_lng_ind][cell_lat_ind]['products'] = [];
				gridMarkers[cell_lng_ind][cell_lat_ind]['products'].push(product);
				//this.addMarker(product);
			}
		}
		
		var bounds = new google.maps.LatLngBounds();
		
		for (var lngName in gridMarkers) {
			for (var latName in gridMarkers[lngName]) {
				var cellText = "",
					lat = 0,
					lng = 0,
					info = "";

				for (var i = 0; i < gridMarkers[lngName][latName]['products'].length; i++) {
					var product = gridMarkers[lngName][latName]['products'][i];
					lat += product.lat;
					lng += product.lng;
					info = '';
					if (product.address && product.address != '')
						info += '<span class="addr">' + product.address + '</span>';
					if (product.phone && product.phone != '')
						info += '<span class="phone">' + product.phone + '</span>';

					cellText += "<span class='product_title'>" + (product.product_url ? "<a href='"+product.product_url +"' target='_blank'>" : "");
					cellText +=  product['product_name'];
					cellText += "</span>" +  ((product.product_url)? "</a>":"");
					cellText += "<p class='product_description'>" + info + "</p>";
				}

				if (gridMarkers[lngName][latName]['products'].length > 0) {
					var marker = {
						'product_name': gridMarkers[lngName][latName]['products'][0]['product_name'],
						'lat': lat/gridMarkers[lngName][latName]['products'].length,
						'lng': lng/gridMarkers[lngName][latName]['products'].length,
						'info': cellText,
						'items_count': gridMarkers[lngName][latName]['products'].length
					};
					this.addMarker(marker);
					bounds.extend(new google.maps.LatLng(marker.lat, marker.lng));
				}
			}
		}

		if(!bounds.isEmpty() && (no_resize === undefined || !no_resize)) {
			this.map.fitBounds(bounds);
		}		
	},
	
	addMarker : function (product) {
		if (!this.map) return;

		var marker = new google.maps.Marker(
			{
				position: new google.maps.LatLng(product.lat, product.lng),
				map: this.map,
				title: (product.items_count < 2)? product.product_name : "" + product.items_count + " items"
			}
		);

		var context = this;
		google.maps.event.addListener(marker, 'click', function() {
			var content = (product.items_count == 1)? product.info : "<p class='items_count'>" + product.items_count + " items</p>" + product.info;
			context.infowindow.setContent("<div class='infowindow'>" + content + "</div>");
			context.infowindow.open(context.map, marker);
		});
		
		this.markers.push(marker);
		this.lats.push(product.lat);
		this.lngs.push(product.lng);
	},
	
	draw : function () {
		var latlng = new google.maps.LatLng(this.defaultLat, this.defaultLng);
		var map_options = {
		  center: latlng,
		  zoom: 18,
		  mapTypeId: google.maps.MapTypeId.ROADMAP
		};
		this.map = new google.maps.Map(document.getElementById("map_canvas"), map_options);

		this.showProductsByOption(this.defaultOption);

		/* Add rebuild markers (regroup) on resize map */
		var context = this;
		google.maps.event.addListener(this.map, 'bounds_changed', function() {
			if(context.zoom !== this.getZoom()) {
				context.zoom = this.getZoom();
				context.showProductsByOption(context.currentOption, false, true);
			}
		});
	}
}